Attribute VB_Name = "Module1"
'---------------------------------------------------------------------------
' This program was developed on a Win2K system using Visual Basic 6.0 for
' windows. The program source and binaries are distributed under a BSD
' Style License.
'
' Developed By:
' Matthew T. Linehan
' American LED-gible Inc.
' Phone: 614.851.1100
' email: engineering@ledgible.com
'
' Copyright (C) 2011, American LED-Gible Inc. All rights reserved.
' Redistribution and use in source and binary forms, with or without
' modification, are permitted provided that the following conditions are met:
'
' Redistributions of source code must retain the above copyright notice, this
' list of conditions, and the following disclaimer. Redistributions in binary
' form must reproduce the above copyright notice, this list of conditions and
' the following disclaimer in the documentation and/or other materials provided
' with the distribution.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
' IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
' ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
' LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
' CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
' SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
' INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
' CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
' ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'---------------------------------------------------------------------------
' V1.0 06-25-2001 Observed Birth PPT Program
' V2.0 05-09-2002 Unknown Large Scale Changes
' V2.1 03-14-2011 Added AutoExport Feature
' V3.0 07-22-2011 Removed Models, Added suppport for Downtime
'---------------------------------------------------------------------------
Option Explicit

'Some constants to make the program more readable
'These are primarily for the integer status variables
Public Const NSCHD As Integer = 96  'Number of Auto Export Schedule Times

'Declare structure for tracking a PPT Register
Type PPTREG
    Value As Long           'Register Value (Read)
    Status As String        'Register Status
End Type

'Declare structure for tracking a single PPT
Type PPTTYPE
    Name As String          'PPT Symbolic Name
    Port As Integer         'Comport 1 to 16 (0 = disabled)
    Address As Integer      'Address character 32 to 126
    Status As String        'PPT Status
    Goal As PPTREG          'Goal Count Register
    Actual As PPTREG        'Actual Count Register
    DownTime As PPTREG      'Down Time Register
    Pace As PPTREG          'Pace Time Register
    GScale As PPTREG        'Goal Scale Register
    AScale As PPTREG        'Acatual Scale Register
    DScale As PPTREG        'DownTime Scale Register
    KHold As PPTREG         'Keypad Hold Register
    LHold As PPTREG         'Logic Hold Register
    KDisable As PPTREG      'KDisable Register
    CmdQueue As Queue       'Pending Commands for this PPT Queue
End Type

'Dim an array to store all data for all PPT's
Public PPT(1 To 64) As PPTTYPE

'Queue for Program to User Messages
Public MsgQueue As New Queue

'Dim an array to store all Auto Export Settings
Public AutoExport(NSCHD) As Integer

'Storage for misc program state
Public OpState As Boolean      'Make false to kill background functions

Public Sub Main()
    
    'Set this to False to kill BackgroundFunction
    OpState = True
    
    'Read our Setup from the registery
    Call ReadConfig
    
    'Bring up the main form
    frmMain.Show
    
    'Start the background task
    'Note this function call does not return
    'until OpState is False
    Call BackgroundFunction
    
    'Write our Setup to the regsistry
    Call WriteConfig
    
    'End the program
    End
    
End Sub
Public Sub BackgroundFunction()
    Dim Unit As Integer
    
    'This function runs in the "Background" and
    'handles all comunications with the PPT's
    'This sub is put in the background by carful use of DoEvents
    'Set OpState = False to kill this sub, or it loops forever
    
    'set what to do on an error
    On Error Resume Next
    
    'setup the Comport
    If (frmMain.MSComm1.PortOpen = True) Then frmMain.MSComm1.PortOpen = False
    frmMain.MSComm1.CommPort = 1                'Use COM1:
    frmMain.MSComm1.Handshaking = comNone       'Disable all handshaking
    frmMain.MSComm1.Settings = "19200,n,8,1"    '19200 baud, no parity, 8 data, 1 stop
    
    'For as long as the programm is running
    While OpState
    
        'Scan all 64 possible units
        For Unit = 1 To 64
            
            ' Yield to the main loop for a bit
            DoEvents
            
            'Check to see if were still running, if not exit loop now
            If Not OpState Then Exit For
            
            'If this unit is active
            If PPT(Unit).Port > 0 Then
            
                'Reprogram the MSComm control to the correct port as required
                If frmMain.MSComm1.CommPort <> PPT(Unit).Port Then
                    If (frmMain.MSComm1.PortOpen = True) Then frmMain.MSComm1.PortOpen = False
                    frmMain.MSComm1.CommPort = PPT(Unit).Port
                End If
                
                'Open the Com port if it's not allready open
                If frmMain.MSComm1.PortOpen = False Then frmMain.MSComm1.PortOpen = True
                    
                'Exchange data with the PPT if the port sucesfully opened
                If frmMain.MSComm1.PortOpen = True Then
                    Call ExchangeData(Unit)
                Else
                    'Comport failed to open
                    'flag PortError on all registers for this PPT
                    With PPT(Unit)
                        .Status = "Port Error"
                        .Goal.Status = "Port Error"
                        .Actual.Status = "Port Error"
                        .DownTime.Status = "Port Error"
                        .Pace.Status = "Port Error"
                        .GScale.Status = "Port Error"
                        .AScale.Status = "Port Error"
                        .DScale.Status = "Prot Error"
                        .KHold.Status = "Port Error"
                        .KDisable.Status = "Port Error"
                        .Goal.Status = "Port Error"
                        .Actual.Status = "Port Error"
                        .Pace.Status = "Port Error"
                        .GScale.Status = "Port Error"
                        .AScale.Status = "Port Error"
                        .KHold.Status = "Port Error"
                        .LHold.Status = "Port Error"
                        .KDisable.Status = "Port Error"
                    End With
                    
                    'tell our forms to update, new data is available
                    Call UpdateUnit(Unit)
                    
                End If
                
            Else 'PPT(Unit).Port > 0
           
                'This PPT is not in use, zero it out
                With PPT(Unit)
                    .Name = "PPT" + Str(Unit)
                    .Address = 0
                    .Status = "(none)"
                    .Goal.Value = 0
                    .Goal.Status = "(none)"
                    .Actual.Value = 0
                    .Actual.Status = "(none)"
                    .DownTime.Value = 0
                    .DownTime.Status = "(none)"
                    .Pace.Value = 0
                    .Pace.Status = "(none)"
                    .GScale.Value = 0
                    .GScale.Status = "(none)"
                    .AScale.Value = 0
                    .AScale.Status = "(none)"
                    .DScale.Value = 0
                    .DScale.Status = "(none)"
                    .KHold.Value = 0
                    .KHold.Status = "(none)"
                    .LHold.Value = 0
                    .LHold.Status = "(none)"
                    .KDisable.Value = 0
                    .KDisable.Status = "(none)"
                End With
                
                'Make sure the Command Queue is Cleared Out
                If (PPT(Unit).CmdQueue.Count > 0) Then PPT(Unit).CmdQueue.Clear
            
                'tell our forms to update
                Call UpdateUnit(Unit)
            
            End If 'PPT(Unit).Port > 0
            
        Next Unit
        
        'Check for AutoExport
        Call CheckAutoExport
            
    Wend 'loop for as long as the program is running
    
    'Make sure the serial port is closed on the way out
    If (frmMain.MSComm1.PortOpen = True) Then frmMain.MSComm1.PortOpen = False
    
End Sub

Public Sub ExchangeData(Unit As Integer)
    
    'This sub manages exchanging data with a single unit
    'Process all pending writes before reading back the new PPT state
    While PPT(Unit).CmdQueue.Count > 0
        Call WriteReg(Unit, PPT(Unit).CmdQueue.Dequeue)
    Wend
    
    'Read back the PPT State
    With PPT(Unit)
        Call ReadReg(Unit, .Goal, "GO")
        Call ReadReg(Unit, .Actual, "AC")
        Call ReadReg(Unit, .DownTime, "DT")
        Call ReadReg(Unit, .Pace, "PA")
        Call ReadReg(Unit, .GScale, "GS")
        Call ReadReg(Unit, .AScale, "AS")
        Call ReadReg(Unit, .DScale, "DS")
        Call ReadReg(Unit, .KHold, "KH")
        Call ReadReg(Unit, .LHold, "LH")
        Call ReadReg(Unit, .KDisable, "KD")
    End With
    
End Sub


Public Sub WriteReg(Unit As Integer, Command As String)
    'This function deals with writing a new value to a PPT register
    Dim STX As String
    Dim Addr As String
    Dim ETX As String
    Dim ACK As String
    Dim NAK As String
    Dim TxBuf As String
    Dim RXBuf As String
    Dim TimeLimit As Single
    
    'Define strings to make the program more readable
    STX = Chr$(2)                   'CTRL-B, Start of transmition
    Addr = Chr(PPT(Unit).Address)   'PPT Address Character
    ETX = Chr$(3)                   'CTRL-C, End of transmition
    ACK = Chr$(6)                   'CTRL-F, Acknowledged
    NAK = Chr$(21)                  'CTRL-U, Not Acknowledged
    
    'Tell the world were working on this PPT
    PPT(Unit).Status = "Writing..."
    Call UpdateUnit(Unit)
    
    'Transmit the write command to the PPT
    TxBuf = STX & Addr & Command & ETX      'Build Full command
    frmMain.MSComm1.Output = TxBuf          'Send Command
    
    'Setup for a 1/5 second timeout
    'Note: Timer is a Single that counts the number of seconds that have
    'elapsed since midnight with about 55mS of accuracy on Win95/98
    'The accuracy of this timer may vary on different operating systems
    'For example, I believe that it's accurate to about 10mS on WinNT/2K
    TimeLimit = Timer + 0.2
    
    'Loop wating for an ACK, NAK or Timeout condition
    While (Timer < TimeLimit And InStr(RXBuf, ACK) = 0 And InStr(RXBuf, NAK) = 0)
        
        'Be nice and yield to the main loop
        'While we wait for a responce from the PPT
        DoEvents
        
        'Accumulate characters recieved via RS-485 in RxBuf
        'Note: Depending on the RS-485 converter used we
        'we may recieve our own transmition as an echo
        If frmMain.MSComm1.InBufferCount Then RXBuf = RXBuf + frmMain.MSComm1.Input
        
    Wend
    
    'HexDump the RXBuf to the status bar
    Call HexDump(RXBuf)
    
    'Now we need to deal with the ACK, NAK, or TimeOut Condition
    If InStr(RXBuf, ACK) > 0 Then
        
        'We have an ACK
        PPT(Unit).Status = "Done"
        Call UpdateUnit(Unit)
        
    ElseIf InStr(RXBuf, NAK) Then
    
            'PPT responed with a NAK
            PPT(Unit).Status = "NAK"
            Call UpdateUnit(Unit)
            Call EnqueueMessage(Unit, "PPT Rejected Command!", Command)
            
    Else
    
            'Timeout
            PPT(Unit).Status = "TIMEOUT"
            Call UpdateUnit(Unit)
            Call EnqueueMessage(Unit, "Timeout Waiting for Response!", Command)
            
    End If

End Sub

Public Sub ReadReg(Unit As Integer, Reg As PPTREG, RegID As String)
    'This function deals with reading a single 6 digit register from a PPT
    Dim STX As String
    Dim Addr As String
    Dim ETX As String
    Dim ACK As String
    Dim NAK As String
    Dim RXBuf As String
    Dim Temp  As String
    Dim TimeLimit As Single
    
    'Define strings to make the program more readable
    STX = Chr$(2)                   'CTRL-B, Start of transmition
    Addr = Chr(PPT(Unit).Address)   'PPT Address Character
    ETX = Chr$(3)                   'CTRL-C, End of transmition
    ACK = Chr$(6)                   'CTRL-F, Acknowledged
    NAK = Chr$(21)                  'CTRL-U, Not Acknowledged
    
    'Tell the world were working on this PPT
    PPT(Unit).Status = "Reading..."
    Reg.Status = "Read Pending..."
    Call UpdateUnit(Unit)
    
    'Transmit the read command to the PPT
    frmMain.MSComm1.Output = STX            'CTRL-B, Start of TX
    frmMain.MSComm1.Output = Addr           'Address
    frmMain.MSComm1.Output = "R"            'Read Command
    frmMain.MSComm1.Output = RegID          'Register ID
    frmMain.MSComm1.Output = ETX            'CTRL-C, End of TX
    
    'Setup for a 1/5 second timeout
    'Note: Timer is a Single that counts the number of seconds that have
    'elapsed since midnight with about 55mS of accuracy on Win95/98
    'The accuracy of this timer may vary on different operating systems
    'For example, I believe that it's accurate to about 10mS on WinNT/2K
    TimeLimit = Timer + 0.2
    
    'Loop wating for an ACK, NAK or Timeout condition
    While (Timer < TimeLimit And InStr(RXBuf, ACK) = 0 And InStr(RXBuf, NAK) = 0)
        
        'Be nice and yield to the main loop
        'While we wait for a responce from the PPT
        DoEvents
        
        'Accumulate characters recieved via RS-485 in RxBuf
        'Note: Depending on the RS-485 converter used we
        'we may recieve our own transmition as an echo
        If frmMain.MSComm1.InBufferCount Then RXBuf = RXBuf + frmMain.MSComm1.Input
        
    Wend
    
    'HexDump the RXBuf to the status bar
    Call HexDump(RXBuf)
    
    'Now we need to deal with the ACK, NAK, or TimeOut Condition
    If InStr(RXBuf, ACK) > 0 Then
        
        'We have an ACK Packet, Extract the Register Value
        Temp = RXBuf
        If InStr(Temp, ETX) Then Temp = Mid(Temp, InStr(Temp, ETX) + 1)
        Reg.Value = Val(Temp)
        Reg.Status = "Done"
        PPT(Unit).Status = "Done"
        Call UpdateUnit(Unit)
        
    ElseIf InStr(RXBuf, NAK) Then
    
            'PPT responed with a NAK
            Reg.Value = -1
            Reg.Status = "NAK"
            PPT(Unit).Status = "NAK"
            Call UpdateUnit(Unit)
            
    Else
    
            'Timeout
            Reg.Status = "TIMEOUT"
            PPT(Unit).Status = "TIMEOUT"
            Call UpdateUnit(Unit)
            
    End If

End Sub


Public Sub UpdateUnit(Unit As Integer)

    'This function is called by Background Function when
    'there is an updated unit in PPT() available
    'our job is to tell the necisary forms to update
    'themseles with the new unit data
    
    'Tell frmMain that PPT(Unit) has updates
    Call frmMain.UpdateData(Unit)
    
    'Tell frmDetail that PPT(Unit) has updates
    If frmDetail.Visible = True Then Call frmDetail.UpdateData(Unit)
    
    'Tell frmWriteReg that PPT(Unit) has updates
    If frmWriteReg.Visible = True Then Call frmWriteReg.UpdateData(Unit)
    
End Sub


Public Sub HexDump(RXBuf As String)

    'This function Hex Dumps the passed string
    'into the status bar. This is used to show
    'a hexdump of the last recieved packet on
    'the status bar. Should help with debuging
    Dim Text As String
    Dim Cnt As Integer
    
    If Val(frmMain.sbStatusBar.Tag) > 0 Then
        Text = "RXBuf: "
        For Cnt = 1 To Len(RXBuf)
            Text = Text + Format(Hex(Asc(Mid(RXBuf, Cnt, 1))), "@@ ")
        Next Cnt
    
        frmMain.sbStatusBar.Panels(1).Text = Text
    End If
    
End Sub


Public Sub ReadConfig()
        
    'Read persistant protion of the config from the registry
    Dim Unit As Integer
    For Unit = 1 To 64
        PPT(Unit).Name = GetSetting(App.Title, "Settings", "Name" + Str(Unit), "PPT" + Str(Unit))
        PPT(Unit).Port = GetSetting(App.Title, "Settings", "Port" + Str(Unit), 0)
        PPT(Unit).Address = GetSetting(App.Title, "Settings", "Address" + Str(Unit), 0)
        
        'Initialize Unit Command Queue
        Set PPT(Unit).CmdQueue = New Queue
        
        'some basic data validation just to be on the safe side
        If Len(PPT(Unit).Name) = 0 Then PPT(Unit).Name = "PPT" + Str(Unit)
        If PPT(Unit).Port < 0 Then PPT(Unit).Port = 0
        If PPT(Unit).Port > 16 Then PPT(Unit).Port = 0
        If PPT(Unit).Address < 33 Then PPT(Unit).Address = 0
        If PPT(Unit).Address > 126 Then PPT(Unit).Address = 0
        
    Next Unit
    
    'Read the Auto Export Schedule from the Registry
    For Unit = 0 To NSCHD - 1
        AutoExport(Unit) = GetSetting(App.Title, "Export", MakeExportTime(Unit), 0)
    Next Unit
    
End Sub


Public Sub WriteConfig()

    'Write the persistant portion of the config from the registry
    Dim Unit As Integer
    For Unit = 1 To 64
        SaveSetting App.Title, "Settings", "Name" + Str(Unit), PPT(Unit).Name
        SaveSetting App.Title, "Settings", "Port" + Str(Unit), PPT(Unit).Port
        SaveSetting App.Title, "Settings", "Address" + Str(Unit), PPT(Unit).Address
    Next Unit

    'Write the Auto Export Schedule to the Registery
    For Unit = 0 To NSCHD - 1
        SaveSetting App.Title, "Export", MakeExportTime(Unit), AutoExport(Unit)
    Next Unit

End Sub

Public Sub CheckAutoExport()
    
    'Note: In visual basic, dates are stored as double precision floating point
    'values that encode the number of days elapsed since 00:00:00 Jan 1 1900
    'Date = Date + 1.0          moves 1 day forward in time
    'Date = Date + (1.0/24)     moves 1 hour forward in time
    'Date = Date + (1.0/1440)   moves 1 minute forward in time
    'Date = Date + (1.0/86400)  moves 1 second forward in time
    
    Dim RTC As Date
    Static CurrentTime As Date
    Static TimeOfDay As Date
    Dim i As Integer
    
    'Get the current time, rounded to the nearest minute
    RTC = CDate(Format(Now, "mm-dd-yyyy hh:mm AM/PM"))
    
    'If this minute is new, check for auto export
    If (RTC <> CurrentTime) Then
    
        'Remember last minute processed
        CurrentTime = RTC
        
        'Index from AutoExport(0) to AutoExport(NSCHD-1)
        For i = 0 To NSCHD - 1
        
            'If this export time is enabled,
            If (AutoExport(i) <> 0) Then
            
                'Check for a time match
                TimeOfDay = Format(CurrentTime, "hh:mm AM/PM")
                If (MakeExportTime(i) = TimeOfDay) Then Call AutoExportCSV(CurrentTime)
                
            End If
            
        Next i
    
    End If
    
End Sub


Public Sub AutoExportCSV(CurrentTime As Date)
    Dim FileName As String
    
    'Construct the File Name YYYYMMDD.csv
    FileName = Format(CurrentTime, "YYYY_MM_DD") & ".CSV"
    
    'Export all data to CSV
    Call ExportCSV(FileName, CurrentTime)
    
End Sub

Public Sub ExportCSV(FileName As String, CurrentTime As Date)
    
    'Open the file out output
    Open FileName For Append As 1
    
    'Append the Data Ledgend if the file is zero length
    If (LOF(1) <= 1) Then
        Write #1, "TIMESTAMP", _
                  "PPT NAME", _
                  "GOAL", _
                  "ACTUAL", _
                  "DOWNTIME", _
                  "PACE", _
                  "GSCALE", _
                  "ASCALE", _
                  "DSCALE", _
                  "KHOLD", _
                  "LHOLD", _
                  "KDISABLE"
    End If
                
    'Write out data for all active units
    Dim Unit As Integer
    For Unit = 1 To 64
        If PPT(Unit).Port > 0 Then
            Write #1, CurrentTime, _
                      PPT(Unit).Name, _
                      PPT(Unit).Goal.Value, _
                      PPT(Unit).Actual.Value, _
                      PPT(Unit).DownTime.Value, _
                      PPT(Unit).Pace.Value, _
                      PPT(Unit).GScale.Value, _
                      PPT(Unit).AScale.Value, _
                      PPT(Unit).DScale.Value, _
                      PPT(Unit).KHold.Value, _
                      PPT(Unit).LHold.Value, _
                      PPT(Unit).KDisable.Value
        End If
    Next Unit
    
    Close 1

End Sub

Public Function MakeExportTime(ByVal i As Integer) As String

    'Construct the export time from the export index
    'index 0  = 12:00AM (Midnight)
    'index 1  = 12:15AM
    'index 2  = 12:30AM
    'index 48 = 12:00PM (12 Noon)
    'index 49 = 12:15PM
    'index 94 = 11:30PM
    'index 95 = 11:45PM
        
    Dim hh As Integer
    Dim mm As Integer
    Dim ap As String
    
    'mm = 00, 15, 30, or 45
    mm = (i Mod 4) * 15
    
    'hh == 1 to 12, AP = AM or PM
    i = i \ 4
    If (i < 12) Then
        ap = "AM"
        hh = i
        If (hh = 0) Then hh = 12
    Else
        ap = "PM"
        hh = i - 12
        If (hh = 0) Then hh = 12
    End If
    
    'Construct Time String
    MakeExportTime = hh & ":" & Format(mm, "00") & ap
    
End Function

Public Function fmt6Digit(Number As Long) As String
    Dim Num As Long
    
    'Format a 6 Digit Register for Display
    Num = Number
    If (Num < 0) Then Num = 0
    If (Num > 999999) Then Num = 999999
    If (Number >= 0) Then
        fmt6Digit = Format(Format(Num, "#####0"), "@@@@@@")
    Else
        fmt6Digit = "------"
    End If
    
End Function


Public Function fmtHHMMSS(Seconds As Long) As String
    Dim Hrs As Long
    Dim Min As Long
    Dim Sec As Long
    
    'Format a Timer Register for Display
    '99:59:59 = 359999 seconds
    'Note: in VB6 backslash is the integer divide operator (no rounding)
    'Note: in VB6 Mod is the modulus operator, returns remainder of integger division
    Sec = Seconds
    If (Sec < 0) Then Sec = 0
    If (Sec > 359999) Then Sec = 359999
    Min = Sec \ 60: Sec = Sec Mod 60
    Hrs = Min \ 60: Min = Min Mod 60
    If (Seconds >= 0) Then
        fmtHHMMSS = Format(Format(Hrs, "#0:") + Format(Min, "00:") + Format(Sec, "00"), "@@@@@@")
    Else
        fmtHHMMSS = "--:--:--"
    End If
    
End Function

Public Function fmtRunHld(Flag As Long) As String
    
    If (Flag <> 0) Then
        fmtRunHld = "HOLD"
    Else
        fmtRunHld = "RUN"
    End If
        
End Function


Public Function fmtEnbDis(Flag As Long) As String

    If (Flag <> 0) Then
        fmtEnbDis = "DISABLED"
    Else
        fmtEnbDis = "ENABLED"
    End If

End Function

Public Function fmtPort(Port As Integer) As String
    
    If (Port >= 1 And Port <= 16) Then
        fmtPort = "Com" + Trim(Str(Port)) + ":"
    Else
        fmtPort = "(none)"
    End If

End Function

Public Function fmtAddr(Addr As Integer) As String
    
    If (Addr >= 33 And Addr <= 126) Then
        fmtAddr = Chr$(Addr) + " (" + Hex(Addr) + "h)"
    Else
        fmtAddr = "(none)"
    End If
    
End Function

Public Function RegVal6Digit(Number As String) As String
    Dim Num As Long
    
    'Interpret User Input as a Number
    If IsNumeric(Number) Then
        Num = Val(Number)
    Else
        Num = 0
    End If
    
    'Clamp result to allowed range
    If (Num < 0) Then Num = 0
    If (Num > 999999) Then Num = 999999
    
    'Convert Back to String in RegVal Format and return to caller
    If (Num <= 9999) Then
        'The 6 Digit PPTs accept both 6 digit and 4 digit RegVals without issue
        'The 4 Digit PPTs will ignore the high 2 digits in a 6 Digit RegVal
        'but the AF-26xx-xxx series reqires the RegVal to be exactly 4 digits
        'So if Num fits in 4 digits, we format it to 4 digits for maximum compatibility
        'If not, we format for 6 digits as a 2nd best option. On 6 Digit PPTs, no problem
        'on 4 Digit PPTs, the high 2 digits will be ignored.  On AF-26xx-xxx the write
        'will be rejected as invalid.
        RegVal6Digit = Format(Num, "0000")
    Else
        RegVal6Digit = Format(Num, "000000")
    End If
    
End Function

Public Function RegValHHMMSS(TimeStr As String) As String
    Dim Seconds As Long
    
    'Interpret user input as a date/time, convert to seconds
    'Note: In VB6 Date/Times are a float, expressing Days Elapsed since 0:00:00 Jan 1 1900
    'Note: There are 86400 Seconds in one Day (24 * 60 * 60)
    If IsDate(TimeStr) Then
        Seconds = CDate(TimeStr) * 86400
    Else
        Seconds = 0
    End If
    
    'Clamp result to allowed range
    'Note There are 359999 Seconds in 99 Hours, 59 Minutes, 59 Seconds
    If (Seconds < 0) Then Seconds = 0
    If (Seconds > 359999) Then Seconds = 359999

    'Convert Back to String in RegVal Format and return to caller
    If (Seconds <= 9999) Then
        'The 6 Digit PPTs accept both 6 digit and 4 digit RegVals without issue
        'The 4 Digit PPTs will ignore the high 2 digits in a 6 Digit RegVal
        'but the AF-26xx-xxx series reqires the RegVal to be exactly 4 digits
        'So if Num fits in 4 digits, we format it to 4 digits for maximum compatibility
        'If not, we format for 6 digits as a 2nd best option. On 6 Digit PPTs, no problem
        'on 4 Digit PPTs, the high 2 digits will be ignored.  On AF-26xx-xxx the write
        'will be rejected as invalid.
        RegValHHMMSS = Format(Seconds, "0000")
    Else
        RegValHHMMSS = Format(Seconds, "000000")
    End If
    
End Function

Public Sub EnqueueMessage(Unit As Integer, Msg As String, Cmd As String)
    'Place Message in Message Queue
    MsgQueue.Enqueue PPT(Unit).Name & vbCrLf & Msg & vbCrLf & "Command = " & Cmd
End Sub

